@@ -2,14 +2,19 @@ import math
 import sys
 
 import py
-from six import binary_type, text_type
 from six.moves import zip, filterfalse
 from more_itertools.more import always_iterable
 
 from _pytest.compat import isclass
+
+from _pytest.compat import Mapping, Sequence
+from _pytest.compat import STRING_TYPES
+
 from _pytest.outcomes import fail
 import _pytest._code
 
+BASE_TYPE = (type, STRING_TYPES)
+
 
 def _cmp_raises_type_error(self, other):
     """__cmp__ implementation which raises TypeError. Used
@@ -20,7 +25,9 @@ def _cmp_raises_type_error(self, other):
     other operators at all.
     """
     __tracebackhide__ = True
-    raise TypeError('Comparison operators other than == and != not supported by approx objects')
+    raise TypeError(
+        "Comparison operators other than == and != not supported by approx objects"
+    )
 
 
 # builtin pytest.approx helper
@@ -47,8 +54,8 @@ class ApproxBase(object):
 
     def __eq__(self, actual):
         return all(
-            a == self._approx_scalar(x)
-            for a, x in self._yield_comparisons(actual))
+            a == self._approx_scalar(x) for a, x in self._yield_comparisons(actual)
+        )
 
     __hash__ = None
 
@@ -79,8 +86,11 @@ class ApproxNumpy(ApproxBase):
         # shape of the array...
         import numpy as np
 
-        return "approx({0!r})".format(list(
-            self._approx_scalar(x) for x in np.asarray(self.expected)))
+        list_scalars = []
+        for x in np.ndindex(self.expected.shape):
+            list_scalars.append(self._approx_scalar(np.asscalar(self.expected[x])))
+
+        return "approx({!r})".format(list_scalars)
 
     if sys.version_info[0] == 2:
         __cmp__ = _cmp_raises_type_error
@@ -94,7 +104,7 @@ class ApproxNumpy(ApproxBase):
             try:
                 actual = np.asarray(actual)
             except:  # noqa
-                raise TypeError("cannot compare '{0}' to numpy.ndarray".format(actual))
+                raise TypeError("cannot compare '{}' to numpy.ndarray".format(actual))
 
         if not np.isscalar(actual) and actual.shape != self.expected.shape:
             return False
@@ -123,9 +133,9 @@ class ApproxMapping(ApproxBase):
     """
 
     def __repr__(self):
-        return "approx({0!r})".format(dict(
-            (k, self._approx_scalar(v))
-            for k, v in self.expected.items()))
+        return "approx({!r})".format(
+            {k: self._approx_scalar(v) for k, v in self.expected.items()}
+        )
 
     def __eq__(self, actual):
         if set(actual.keys()) != set(self.expected.keys()):
@@ -147,8 +157,9 @@ class ApproxSequence(ApproxBase):
         seq_type = type(self.expected)
         if seq_type not in (tuple, list, set):
             seq_type = list
-        return "approx({0!r})".format(seq_type(
-            self._approx_scalar(x) for x in self.expected))
+        return "approx({!r})".format(
+            seq_type(self._approx_scalar(x) for x in self.expected)
+        )
 
     def __eq__(self, actual):
         if len(actual) != len(self.expected):
@@ -163,6 +174,7 @@ class ApproxScalar(ApproxBase):
     """
     Perform approximate comparisons for single numbers only.
     """
+
     DEFAULT_ABSOLUTE_TOLERANCE = 1e-12
     DEFAULT_RELATIVE_TOLERANCE = 1e-6
 
@@ -184,14 +196,14 @@ class ApproxScalar(ApproxBase):
         # If a sensible tolerance can't be calculated, self.tolerance will
         # raise a ValueError.  In this case, display '???'.
         try:
-            vetted_tolerance = '{:.1e}'.format(self.tolerance)
+            vetted_tolerance = "{:.1e}".format(self.tolerance)
         except ValueError:
-            vetted_tolerance = '???'
+            vetted_tolerance = "???"
 
         if sys.version_info[0] == 2:
-            return '{0} +- {1}'.format(self.expected, vetted_tolerance)
+            return "{} +- {}".format(self.expected, vetted_tolerance)
         else:
-            return u'{0} \u00b1 {1}'.format(self.expected, vetted_tolerance)
+            return u"{} \u00b1 {}".format(self.expected, vetted_tolerance)
 
     def __eq__(self, actual):
         """
@@ -232,6 +244,7 @@ class ApproxScalar(ApproxBase):
         absolute tolerance or a relative tolerance, depending on what the user
         specified or which would be larger.
         """
+
         def set_default(x, default):
             return x if x is not None else default
 
@@ -240,7 +253,9 @@ class ApproxScalar(ApproxBase):
         absolute_tolerance = set_default(self.abs, self.DEFAULT_ABSOLUTE_TOLERANCE)
 
         if absolute_tolerance < 0:
-            raise ValueError("absolute tolerance can't be negative: {}".format(absolute_tolerance))
+            raise ValueError(
+                "absolute tolerance can't be negative: {}".format(absolute_tolerance)
+            )
         if math.isnan(absolute_tolerance):
             raise ValueError("absolute tolerance can't be NaN.")
 
@@ -255,10 +270,14 @@ class ApproxScalar(ApproxBase):
         # we've made sure the user didn't ask for an absolute tolerance only,
         # because we don't want to raise errors about the relative tolerance if
         # we aren't even going to use it.
-        relative_tolerance = set_default(self.rel, self.DEFAULT_RELATIVE_TOLERANCE) * abs(self.expected)
+        relative_tolerance = set_default(
+            self.rel, self.DEFAULT_RELATIVE_TOLERANCE
+        ) * abs(self.expected)
 
         if relative_tolerance < 0:
-            raise ValueError("relative tolerance can't be negative: {}".format(absolute_tolerance))
+            raise ValueError(
+                "relative tolerance can't be negative: {}".format(absolute_tolerance)
+            )
         if math.isnan(relative_tolerance):
             raise ValueError("relative tolerance can't be NaN.")
 
@@ -269,8 +288,8 @@ class ApproxScalar(ApproxBase):
 class ApproxDecimal(ApproxScalar):
     from decimal import Decimal
 
-    DEFAULT_ABSOLUTE_TOLERANCE = Decimal('1e-12')
-    DEFAULT_RELATIVE_TOLERANCE = Decimal('1e-6')
+    DEFAULT_ABSOLUTE_TOLERANCE = Decimal("1e-12")
+    DEFAULT_RELATIVE_TOLERANCE = Decimal("1e-6")
 
 
 def approx(expected, rel=None, abs=None, nan_ok=False):
@@ -426,8 +445,6 @@ def approx(expected, rel=None, abs=None, nan_ok=False):
        __ https://docs.python.org/3/reference/datamodel.html#object.__ge__
     """
 
-    from collections import Mapping, Sequence
-    from _pytest.compat import STRING_TYPES as String
     from decimal import Decimal
 
     # Delegate the comparison to a class that knows how to deal with the type
@@ -448,7 +465,7 @@ def approx(expected, rel=None, abs=None, nan_ok=False):
         cls = ApproxNumpy
     elif isinstance(expected, Mapping):
         cls = ApproxMapping
-    elif isinstance(expected, Sequence) and not isinstance(expected, String):
+    elif isinstance(expected, Sequence) and not isinstance(expected, STRING_TYPES):
         cls = ApproxSequence
     elif isinstance(expected, Decimal):
         cls = ApproxDecimal
@@ -466,9 +483,10 @@ def _is_numpy_array(obj):
     import inspect
 
     for cls in inspect.getmro(type(obj)):
-        if cls.__module__ == 'numpy':
+        if cls.__module__ == "numpy":
             try:
                 import numpy as np
+
                 return isinstance(obj, np.ndarray)
             except ImportError:
                 pass
@@ -478,8 +496,9 @@ def _is_numpy_array(obj):
 
 # builtin pytest.raises helper
 
+
 def raises(expected_exception, *args, **kwargs):
-    """
+    r"""
     Assert that a code block/function call raises ``expected_exception``
     and raise a failure exception otherwise.
 
@@ -585,13 +604,14 @@ def raises(expected_exception, *args, **kwargs):
 
     """
     __tracebackhide__ = True
-    base_type = (type, text_type, binary_type)
-    for exc in filterfalse(isclass, always_iterable(expected_exception, base_type)):
-        msg = ("exceptions must be old-style classes or"
-               " derived from BaseException, not %s")
+    for exc in filterfalse(isclass, always_iterable(expected_exception, BASE_TYPE)):
+        msg = (
+            "exceptions must be old-style classes or"
+            " derived from BaseException, not %s"
+        )
         raise TypeError(msg % type(exc))
 
-    message = "DID NOT RAISE {0}".format(expected_exception)
+    message = "DID NOT RAISE {}".format(expected_exception)
     match_expr = None
 
     if not args:
@@ -600,8 +620,8 @@ def raises(expected_exception, *args, **kwargs):
         if "match" in kwargs:
             match_expr = kwargs.pop("match")
         if kwargs:
-            msg = 'Unexpected keyword arguments passed to pytest.raises: '
-            msg += ', '.join(kwargs.keys())
+            msg = "Unexpected keyword arguments passed to pytest.raises: "
+            msg += ", ".join(kwargs.keys())
             raise TypeError(msg)
         return RaisesContext(expected_exception, message, match_expr)
     elif isinstance(args[0], str):
